/* USER CODE BEGIN Header */
/**
 *
 * By: 			Enrico Marinoni
 * Date:		02/October/2021
 * Made for:	NUCLEO-F401RE
 * Internet:    https://www.emcu-homeautomation.org/professional-control-system-for-control-the-water-level-dark-waters-in-home-assistant-zigbee/
 * Description:
 * 	This SW is fundamentally a debounce control for:
	float contact for water level
	There is the possibility to set the debounce from 1 to 20 sec
	for avoid the water fluctuations.
	In practice there is an analog and digital filter for the
	INPUT signal and the output drive a relay (see the schematics).
 *
 *
 *
 *
 *
 *
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define Debounce 100	// 100 mSec
#define LongDly 30000	// 30 Sec

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */
// Variables
uint16_t Cnf0 = 1;
uint16_t Cnf1 = 1;
uint16_t Dly  = 1000;
uint8_t  RelayStatus   = 10;	// 0 == OFF
								// 1 == ON
								// Other value means undefined
uint8_t  InPutH2O_1    = 10; 	// First  reading  of InPutH2O
								//        0 == No Water   1 == Water present
uint8_t  InPutH2O_2    = 20; 	// Second reading after debounce time of InPutH2O
								//        0 == No Water   1 == Water present

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_USART2_UART_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USART2_UART_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

	  // Read the status of Cnf0 (PC1) & Cnf1 (PB0
	  Cnf0 = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_0);
	  Cnf1 = HAL_GPIO_ReadPin(GPIOC, GPIO_PIN_1);

	  /* Calculate the Dly (delay) according to input of Cnf0 & Cnf1
	  	  Cnf0	Cnf1	Dly 	== Sec.
	  	  1		1		5000	 	5	  (default, no jumper to GND)
	  	  1		0		1000	 	1
	  	  0		1		10000	 	10
	  	  0		0		20000	 	20
	  */
	  if ((Cnf0==1) & (Cnf1==1))
		  Dly = 5000;
	  if ((Cnf0==1) & (Cnf1==0))
		  Dly = 1000;
	  if ((Cnf0==0) & (Cnf1==1))
		  Dly = 10000;
	  if ((Cnf0==0) & (Cnf1==0))
		  Dly = 20000;

	  /* Read the status of the InPut pin (PA4) Sensor of H2O
	   	   0 == No Water (Close)
	   	   1 == Water    (Open)
	  */
	  InPutH2O_1 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);
	  HAL_Delay(Debounce);
	  InPutH2O_2 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);

	  /* Drive the Relay according to InPutH2O
	    InPutH2O	Relay
	   	   1		ON		ON  pump	- Water present
	   	   0		OFF		OFF pump	- NO Water
	   */
	  if ((InPutH2O_1 == 1) & (InPutH2O_2 == 1))
	  	  {
		  if (RelayStatus != 1)
		  	  {
			  // Delay
			  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, 1);
			  HAL_Delay(Dly);
			  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, 0);
			  // Read again the InPut pin (PA4) Sensor of H2O
			  InPutH2O_1 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);
			  HAL_Delay(Debounce);
			  InPutH2O_2 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);
			  // Test again the the Sensor of H2O and if is already confirmed drive pump (ON)
			  if ((InPutH2O_1 == 1) & (InPutH2O_2 == 1))
				  {
				  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_0, 1); // ON  pump
				  RelayStatus = 1;
				  // Long Dly after the Relay ON
				  HAL_Delay(LongDly);
				  HAL_Delay(LongDly);
				  HAL_Delay(LongDly);
				  HAL_Delay(LongDly);
				  }
			  }
	  	  }
	  else
		  {
		  if ((InPutH2O_1 == 0) & (InPutH2O_2 == 0))
			  {
			  if (RelayStatus != 0)
				  {
				  // Delay
				  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, 1);
				  HAL_Delay(Dly);
				  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, 0);
				  // Read again the InPut pin (PA4) Sensor of H2O
				  InPutH2O_1 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);
				  HAL_Delay(Debounce);
				  InPutH2O_2 = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_4);
				  // Test again the the Sensor of H2O and if is already confirmed OFF the pump
				  if ((InPutH2O_1 == 0) & (InPutH2O_2 == 0))
					  {
					  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_0, 0); // OFF pump
					  RelayStatus = 0;
					  // Long Dly after the Relay OFF
					  HAL_Delay(LongDly);
					  HAL_Delay(LongDly);
					  HAL_Delay(LongDly);
					  HAL_Delay(LongDly);
					  }
				  }
			  }
		  }
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);
  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLM = 8;
  RCC_OscInitStruct.PLL.PLLN = 72;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV6;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(RELE_GPIO_Port, RELE_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : B1_Pin */
  GPIO_InitStruct.Pin = B1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : RELE_Pin */
  GPIO_InitStruct.Pin = RELE_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(RELE_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : Cnf0_Pin */
  GPIO_InitStruct.Pin = Cnf0_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(Cnf0_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : WaterContact_Pin */
  GPIO_InitStruct.Pin = WaterContact_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(WaterContact_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : LD2_Pin */
  GPIO_InitStruct.Pin = LD2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LD2_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : Cnf1_Pin */
  GPIO_InitStruct.Pin = Cnf1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(Cnf1_GPIO_Port, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
